package visualization6;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;

/**
 * 領域の2分割を再帰的に繰り返し、ツリーマップの描画を行います。
 */

public class BinaryTreeMapRenderer implements TreeMapRenderer {
    public void render(Graphics2D g, Node node, Rectangle2D bounds) {
        doRender(g, node, bounds, 0);
    }

    private void doRender(Graphics2D g, Node node, Rectangle2D bounds, int depth) {
        if (node instanceof BookmarkItem) {
            BookmarkItem item = (BookmarkItem) node;
            // 領域を塗りつぶす
            g.setPaint(bookmarkToColor(item));
            g.fill(bounds);
            // ブックマークタイトルを描画
            g.setFont(new Font("Dialog", Font.BOLD, 12));
            g.setPaint(Color.WHITE);
            drawStringWithin(g, item.getName(), bounds);
        } else if (node instanceof Cluster) {
            Cluster cluster = (Cluster) node;

            // 子ノードchild1とchild2を取得
            Node child1 = cluster.getLeft();
            Node child2 = cluster.getRight();
            // child1の面積の方が大きくなるようにする
            if (child1.getArea() < child2.getArea()) {
                Node temp = child1;
                child1 = child2;
                child2 = temp;
            }

            // 子ノードの面積比を計算
            double area1 = child1.getArea();
            double area2 = child2.getArea();
            double ratio1 = area1 / (area1 + area2);
            double ratio2 = area2 / (area1 + area2);

            double x = bounds.getX();
            double y = bounds.getY();
            double w = bounds.getWidth();
            double h = bounds.getHeight();

            Rectangle2D rect1;
            Rectangle2D rect2;
            // 領域分割を実行
            if (w > h) {
                // boundsが横長の場合、左右に分割
                rect1 = new Rectangle2D.Double(x, y, ratio1 * w, h);
                rect2 = new Rectangle2D.Double(x + ratio1 * w, y, ratio2 * w, h);
            } else {
                // boundsが縦長の場合、上下に分割
                rect1 = new Rectangle2D.Double(x, y, w, ratio1 * h);
                rect2 = new Rectangle2D.Double(x, y + ratio1 * h, w, ratio2 * h);
            }
            // 子ノードを再帰的に処理する
            doRender(g, child1, rect1, depth + 1);
            doRender(g, child2, rect2, depth + 1);
        }

        // 輪郭を階層の深さに応じた太さで描画
        float borderWidth = Math.max(8f - depth, 1f);
        g.setStroke(new BasicStroke(borderWidth));
        g.setPaint(Color.BLACK);
        g.draw(bounds);
    }

    /**
     * ブックマークノードを色に変換します。
     * 
     * @param item
     *            ブックマークノード
     * @return 色
     */
    private Color bookmarkToColor(BookmarkItem item) {
        BookmarkDetail detail = item.getDetail();
        // コメント率を計算
        float commentRate = (float) detail.comments.size()
                / (float) detail.bookmarkCount;
        // 補正をかける
        commentRate = Math.min(2f * commentRate, 1f);
        // コメント率からRGB要素を決定
        float red = 0.2f + 0.4f * commentRate;
        float green = 0.2f + 0.4f * (1f - commentRate);
        float blue = 0.2f;
        return new Color(red, green, blue);
    }

    private void drawStringWithin(Graphics2D g, String s, Rectangle2D rect) {
        AttributedString str = new AttributedString(s);
        str.addAttribute(TextAttribute.FONT, g.getFont());
        AttributedCharacterIterator iterator = str.getIterator();
        FontRenderContext context = g.getFontRenderContext();
        LineBreakMeasurer measurer = new LineBreakMeasurer(iterator, context);
        float padding = 8f;
        float width = Math.max((float) rect.getWidth() - 2f * padding, 1f);
        float y = (float) rect.getY() + padding;
        while (measurer.getPosition() < s.length()) {
            TextLayout layout = measurer.nextLayout(width);
            Rectangle2D bounds = layout.getBounds();
            y += (float) bounds.getHeight() + 2f;
            layout.draw(g, (float) rect.getX() + padding, y);
        }
    }
}
