package visualization6;

/**
 * 任意次元のベクトルです。
 */

public class MultiVector {
    private double[] data;

    /**
     * ゼロベクトルを作成します。
     * 
     * @param dimension
     *            次元
     */
    public MultiVector(int dimension) {
        data = new double[dimension];
    }

    /**
     * 成分の配列からベクトルを作成します。
     * 
     * @param data
     *            成分配列
     */
    public MultiVector(double... data) {
        this.data = data;
    }

    /**
     * ベクトルの次元を取得します。
     * 
     * @return 次元
     */
    public int dimension() {
        return data.length;
    }

    /**
     * 指定された次数に対応するベクトル成分を取得します。
     * 
     * @param index
     *            次数
     * @return 成分
     */
    public double get(int index) {
        return data[index];
    }

    /**
     * ベクトル成分を設定します。
     * 
     * @param index
     *            次数
     * @param value
     *            成分
     */
    public void set(int index, double value) {
        data[index] = value;
    }

    /**
     * ベクトルのコピーを作成します。
     * 
     * @return コピーされたベクトル
     */
    public MultiVector copy() {
        int dim = dimension();
        MultiVector v = new MultiVector(dim);
        for (int i = 0; i < dim; i++) {
            v.data[i] = data[i];
        }
        return v;
    }

    private void checkDimension(MultiVector v) {
        if (dimension() != v.dimension()) {
            throw new IllegalArgumentException("Dimension mismatch.");
        }
    }

    /**
     * 指定されたベクトルを加算します。
     * 
     * @param v
     *            ベクトル
     */
    public void add(MultiVector v) {
        checkDimension(v);
        for (int i = 0; i < dimension(); i++) {
            data[i] += v.data[i];
        }
    }

    /**
     * 指定されたベクトルを減算します。
     * 
     * @param v
     *            ベクトル
     */
    public void subtract(MultiVector v) {
        checkDimension(v);
        for (int i = 0; i < dimension(); i++) {
            data[i] -= v.data[i];
        }
    }

    /**
     * 指定された数値でベクトルを定数倍します。
     * 
     * @param d
     *            乗数
     */
    public void multiply(double d) {
        for (int i = 0; i < dimension(); i++) {
            data[i] *= d;
        }
    }

    /**
     * 指定された数値でベクトルを除算します。
     * 
     * @param d
     *            除数
     */
    public void divide(double d) {
        multiply(1.0 / d);
    }

    /**
     * ベクトルのノルム（長さ）を取得します。
     * 
     * @return ノルム
     */
    public double norm() {
        double sum = 0;
        for (int i = 0; i < data.length; i++) {
            double d = data[i];
            sum += d * d;
        }
        return Math.sqrt(sum);
    }

    /**
     * ベクトルを正規化します（ノルムを1にします）。
     */
    public void normalize() {
        divide(norm());
    }

    /**
     * 指定されたベクトルとの距離の二乗を取得します。
     * 
     * @param v
     *            ベクトル
     * @return 距離の二乗
     */
    public double distanceSq(MultiVector v) {
        checkDimension(v);
        double sum = 0;
        for (int i = 0; i < data.length; i++) {
            double d = v.data[i] - data[i];
            sum += d * d;
        }
        return sum;
    }
}
